package edu.travel.tenant.web;
import cn.hutool.core.bean.BeanUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.remote.feign.mode.dto.tenant.AddOrUpdateRoleDto;
import edu.travel.remote.feign.mode.dto.tenant.AssignRolesOrMenusDto;
import edu.travel.remote.feign.mode.dto.tenant.SysRolePageDto;
import edu.travel.remote.feign.mode.vo.tenant.MenuTreeVo;
import edu.travel.remote.feign.mode.vo.tenant.SysRoleVo;
import edu.travel.resp.BaseResponse;
import edu.travel.resp.PageResponse;
import edu.travel.tenant.SysRoleRemoteController;
import edu.travel.tenant.entity.EduTenant;
import edu.travel.tenant.entity.SysRole;
import edu.travel.tenant.service.ISysRoleService;
import edu.travel.web.BaseController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * SysRoleController 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/2/6
 */

@RestController
@RequestMapping("/sysRole")
public class SysRoleController extends BaseController<SysRole> implements SysRoleRemoteController {

    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 新增管理角色并分配菜单
     */
    @PostMapping("/addRoleAndAssignMenus")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<SysRoleVo> addRoleAndAssignMenus(@RequestBody AddOrUpdateRoleDto addOrUpdateRoleDto) {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        SysRole role = sysRoleService.addRoleAndAssignMenus(addOrUpdateRoleDto, principal);
        return new BaseResponse<>(200,"success",BeanUtil.copyProperties(role,SysRoleVo.class));
    }

    /**
     * 删除角色多个
     */
    @PostMapping("/deleteRoleByIds")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> deleteRoleByIds(@RequestBody List<Long> ids) {
        HashSet<Long> longs = new HashSet<>(ids);
        sysRoleService.deleteByIds(longs);
        return new BaseResponse<>(200,"success",true) ;
    }

    /**
     * 删除角色单个
     */
    @PostMapping("/deleteRoleById")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> deleteRoleById(@RequestParam Long id) {
        Set<Long> list = new HashSet<>();
        list.add(id);
        sysRoleService.deleteByIds(list);
        return new BaseResponse<>(200,"success",true) ;
    }

    /**
     * 更新角色和角色下菜单
     */
    @PostMapping("/updateRoleAndMenusById")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> updateRoleAndMenusById( @RequestBody AddOrUpdateRoleDto addOrUpdateRoleDto) {
        sysRoleService.updateRoleAndMenusById(addOrUpdateRoleDto);
        return new BaseResponse<>(200,"success",true) ;
    }

    /**
     * 为角色分配菜单
     * @param dto
     * @return
     */
    @PostMapping("/assignMenusToRole")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> assignMenusToRole(@RequestBody AssignRolesOrMenusDto dto) {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        sysRoleService.assignMenusToRole(dto,principal);
        return new BaseResponse<>(200,"sucess",true);
    }

    /**
     * 更新角色菜单
     * @param assignRolesOrMenusDto
     * @return
     */
    @PostMapping("/updateMenusToRole")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<Boolean> updateMenusToRole(@RequestBody AssignRolesOrMenusDto assignRolesOrMenusDto) {
        EduTenant principal = (EduTenant) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        sysRoleService.updateMenusToRole(assignRolesOrMenusDto,principal);
        return PageResponse.out(200,"success",true);
    }

    /**
     * 分页查询角色列表
     * @param sysRolePageDto
     * @return
     */
    @GetMapping("/getRolePageList")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<List<SysRoleVo>> getRolePageList(SysRolePageDto sysRolePageDto) {
        IPage<SysRole> page = new Page<>(sysRolePageDto.getPageNum(), sysRolePageDto.getPageSize());
        IPage<SysRole> tenantIPage = sysRoleService.page(page, new LambdaQueryWrapper<SysRole>().eq(SysRole::getDeleteFlag, 0)
                .like(StringUtils.isNotBlank(sysRolePageDto.getName()),SysRole::getName, sysRolePageDto.getName()));
        List<SysRoleVo> sysRoleVos = BeanUtil.copyToList(tenantIPage.getRecords(), SysRoleVo.class);
        return PageResponse.out(200, "success", sysRoleVos, (int) tenantIPage.getTotal(), (int) tenantIPage.getSize());
    }

    /**
     * 根据角色id查询菜单树
     * @param roleId
     * @return
     */
    @GetMapping("/getMenuTreeByRoleId")
//    @PreAuthorize("hasRole('超级管理员')")
    public BaseResponse<List<MenuTreeVo>> getMenuTreeByRoleId( Long roleId) {
        List<MenuTreeVo> menuTreeVos = sysRoleService.getMenuTreeByRoleId(roleId);
        return PageResponse.out(200, "success", menuTreeVos);
    }
}
